@@ -21,18 +21,24 @@ from db.base.models import DemodData, LatestTleSet, Mode, Satellite, Tle, Transm
 LOGGER = logging.getLogger('db')
 
 
-def remove_latest_tle_set(satellite_id):
+def remove_latest_tle_set(satellite_pk):
     """Remove LatestTleSet entry for specific Satellite"""
-    LatestTleSet.objects.filter(satellite__pk=satellite_id).delete()
+    LatestTleSet.objects.filter(satellite__pk=satellite_pk).delete()
 
 
-def update_latest_tle_sets(satellite_ids=None):
+def update_latest_tle_sets(satellite_pks=None):
     """Update LatestTleSet entries for all or specific Satellites"""
     # Select satellite models
-    if satellite_ids:
-        satellites = Satellite.objects.filter(pk__in=satellite_ids)
+    if satellite_pks:
+        satellites = Satellite.objects.filter(
+            pk__in=satellite_pks,
+            associated_satellite__isnull=True,
+            satellite_entry__approved=True
+        ).exclude(satellite_entry__status__in=['re-entered', 'future'])
     else:
-        satellites = Satellite.objects.exclude(status__in=['re-entered', 'future'])
+        satellites = Satellite.objects.filter(
+            associated_satellite__isnull=True, satellite_entry__approved=True
+        ).exclude(satellite_entry__status__in=['re-entered', 'future'])
 
     # Create dictionary with Satellite ids as keys and Tle ids as values
     latest_triplets = LatestTleSet.objects.filter(
@@ -141,7 +147,9 @@ def calculate_statistics():
     :returns: a dictionary of statistics
     """
     # satellite statistics
-    satellites = Satellite.objects.all()
+    satellites = Satellite.objects.filter(
+        associated_satellite__isnull=True, satellite_entry__approved=True
+    )
     total_satellites = satellites.count()
 
     # data statistics
@@ -175,10 +183,11 @@ def calculate_statistics():
 def calculate_transmitters_stats():
     """Helper function to provite transmitters and statistics about
     transmitters in db (such as total and percentage of alive)"""
-    transmitters = Transmitter.objects.all()
+    transmitters = Transmitter.objects.filter(
+        satellite__associated_satellite__isnull=True, satellite__satellite_entry__approved=True
+    )
     total_transmitters = transmitters.count()
-    # Remove the following pylint disable after Python 3 migration
-    alive_transmitters = transmitters.filter(status='active').count()  # pylint: disable=E1101
+    alive_transmitters = transmitters.filter(status='active').count()
     if alive_transmitters > 0 and total_transmitters > 0:
         try:
             alive_transmitters_percentage = '{0}%'.format(
@@ -309,9 +318,9 @@ def create_point(fields, satellite, telemetry, demoddata, version):
     point = [
         {
             'time': demoddata.timestamp.strftime('%Y-%m-%dT%H:%M:%SZ'),
-            'measurement': satellite.norad_cat_id,
+            'measurement': satellite.satellite_entry.norad_cat_id,
             'tags': {
-                'satellite': satellite.name,
+                'satellite': satellite.satellite_entry.name,
                 'decoder': telemetry.decoder,
                 'station': demoddata.station,
                 'observer': demoddata.observer,
@@ -374,29 +383,41 @@ def decode_demoddata(demoddata, satellite, tlmdecoder):
         LOGGER.error(error, exc_info=True)
 
 
-def decode_data(norad_id, demoddata_id=None, redecode=False):
+def decode_data(sat_id, demoddata_id=None, redecode=False):
     """Decode data for a satellite, with an option to limit the scope.
 
-    :param norad_id: the NORAD ID of the satellite to decode data for
+    :param sat_id: the Satellite Identifier of the satellite to decode data for
     :param demoddata_id: if demoddata_id exists, try to decode this demoddata object
     :param redecode: redecode demoddata or only recent
     """
-    satellite = Satellite.objects.prefetch_related('telemetries').get(norad_cat_id=norad_id)
+    satellite = Satellite.objects.get(satellite_identifier__sat_id=sat_id)
+
+    # If satellite is merged use telemetries from the associated satellite
+    if satellite.associated_satellite:
+        satellite = satellite.associated_satellite
     telemetry_decoders = satellite.telemetries.all()
+
     if telemetry_decoders:
         data = DemodData.objects.select_for_update()
-        if redecode:
-            if demoddata_id:
+        if demoddata_id:
+            if redecode:
                 data = data.filter(pk=demoddata_id)
             else:
-                data = data.filter(satellite__norad_cat_id=norad_id)
-        else:
-            if demoddata_id:
                 data = data.filter(pk=demoddata_id, is_decoded=False)
+        else:
+            # Get a list of all associated satellites for decoding all related data
+            associated_satellites = list(
+                satellite.associated_with.all().values_list('pk', flat=True)
+            )
+            satellites = associated_satellites.append(satellite.pk)
+            if redecode:
+                data = data.filter(satellite_identifier__sat_id__in=satellites)
             else:
                 time_period = make_aware(datetime.utcnow() - timedelta(hours=48))
                 data = data.filter(
-                    satellite__norad_cat_id=norad_id, timestamp__gte=time_period, is_decoded=False
+                    satellite_identifier__sat_id__in=satellites,
+                    timestamp__gte=time_period,
+                    is_decoded=False
                 )
 
         with transaction.atomic():
@@ -416,25 +437,64 @@ def cache_statistics():
     statistics = calculate_statistics()
     cache.set('stats_transmitters', statistics, 60 * 60 * 2)
 
-    ids = Satellite.objects.values('id')
+    ids = []
+    sat_stats = {}
+
+    satellites = Satellite.objects.filter(satellite_entry__approved=True).values(
+        'satellite_entry__name', 'satellite_entry__norad_cat_id', 'id', 'associated_satellite',
+        'satellite_identifier__sat_id'
+    ).annotate(
+        count=Count('telemetry_data'),
+        decoded=Count('telemetry_data', filter=Q(telemetry_data__is_decoded=True)),
+        latest_payload=Max('telemetry_data__timestamp')
+    )
+
+    # Aggregate stats for satellites and their associated ones
+    for sat in satellites:
+        # if satellite is merged then add statistics to its association
+        if sat['associated_satellite']:
+            if sat['associated_satellite'] in sat_stats:
+                sat_stats[sat['associated_satellite']]['count'] += sat['count']
+                sat_stats[sat['associated_satellite']]['decoded'] += sat['decoded']
+                sat_stats[sat['associated_satellite']]['latest_payload'] = max(
+                    sat_stats[sat['associated_satellite']]['latest_payload'],
+                    sat['latest_payload'],
+                    default=sat['latest_payload']
+                )
+            else:
+                del sat['associated_satellite']
+                sat_stats[sat['associated_satellite']] = sat
+        else:
+            ids.append(sat['id'])
+            # if non-merged satellite is already in sat_stats then overwrite
+            # the name and the NORAD ID as the current ones are from one of its
+            # associated satellites that have been merged with it.
+            if sat['id'] in sat_stats:
+                sat_stats[sat['id']]['satellite_entry__name'] = sat['satellite_entry__name']
+                sat_stats[sat['id']
+                          ]['satellite_entry__norad_cat_id'] = sat['satellite_entry__norad_cat_id']
+                sat_stats[sat['id']
+                          ]['satellite_identifier__sat_id'] = sat['satellite_identifier__sat_id']
+                sat_stats[sat['id']]['count'] += sat['count']
+                sat_stats[sat['id']]['decoded'] += sat['decoded']
+                if sat_stats[sat['id']]['latest_payload']:
+                    if sat['latest_payload']:
+                        sat_stats[sat['id']]['latest_payload'] = max(
+                            sat_stats[sat['id']]['latest_payload'], sat['latest_payload']
+                        )
+                else:
+                    sat_stats[sat['id']]['latest_payload'] = sat['latest_payload']
+            else:
+                del sat['associated_satellite']
+                sat_stats[sat['id']] = sat
+
+    for sat_pk in sat_stats:
+        cache.set(sat_pk, sat_stats[sat_pk], 60 * 60 * 2)
     cache.set('satellites_ids', ids, 60 * 60 * 2)
 
-    satellites = Satellite.objects \
-                          .values('name', 'norad_cat_id', 'id') \
-                          .annotate(count=Count('telemetry_data'),
-                                    decoded=Count('telemetry_data',
-                                                  filter=Q(telemetry_data__is_decoded=True)),
-                                    latest_payload=Max('telemetry_data__timestamp')) \
-                          .order_by('-count')
-
-    for sat in satellites:
-        cache.set(sat['id'], sat, 60 * 60 * 2)
-
-    observers = DemodData.objects \
-                         .values('observer') \
-                         .annotate(count=Count('observer'),
-                                   latest_payload=Max('timestamp')) \
-                         .order_by('-count')
+    observers = DemodData.objects.values('observer').annotate(
+        count=Count('observer'), latest_payload=Max('timestamp')
+    ).order_by('-count')
     cache.set('stats_observers', observers, 60 * 60 * 2)
 
 
@@ -481,8 +541,8 @@ def read_influx(norad):
         # epoch:s to set the return timestamp in unixtime for easier conversion
         params = {'epoch': 's'}
         results = client.query(
-            'SELECT count(*) FROM "' + str(norad) +
-            '" WHERE time > now() - 30d GROUP BY time(1d) fill(null)',
+            'SELECT count(*) FROM "' + str(norad)
+            + '" WHERE time > now() - 30d GROUP BY time(1d) fill(null)',
             params=params
         )
         return results.raw
