@@ -7,7 +7,7 @@ from django.conf import settings
 from django.contrib import messages
 from django.contrib.auth import get_user_model
 from django.contrib.auth.decorators import login_required
-from django.contrib.auth.mixins import LoginRequiredMixin, PermissionRequiredMixin
+from django.contrib.auth.mixins import LoginRequiredMixin
 from django.core.cache import cache
 from django.core.exceptions import ObjectDoesNotExist
 from django.core.paginator import Paginator
@@ -19,11 +19,12 @@ from django.urls import reverse
 from django.utils.timezone import now
 from django.views.decorators.http import require_POST
 
-from db.base.forms import SatelliteModelForm, TransmitterModelForm, TransmitterUpdateForm
+from db.base.forms import SatelliteCreateForm, SatelliteUpdateForm, TransmitterCreateForm, \
+    TransmitterUpdateForm
 from db.base.helpers import get_apikey
-from db.base.models import DemodData, Satellite, Transmitter, TransmitterEntry, \
-    TransmitterSuggestion
-from db.base.tasks import export_frames, notify_transmitter_suggestion
+from db.base.models import DemodData, Satellite, SatelliteEntry, SatelliteIdentifier, \
+    SatelliteSuggestion, Transmitter, TransmitterEntry, TransmitterSuggestion
+from db.base.tasks import export_frames, notify_suggestion
 from db.base.utils import cache_statistics, millify, read_influx
 
 LOGGER = logging.getLogger('db')
@@ -43,9 +44,9 @@ def home(request):
         to_attr='suggested_transmitters'
     )
 
-    newest_sats = Satellite.objects.all().order_by('-id')[:5].prefetch_related(
-        prefetch_approved, prefetch_suggested
-    )
+    newest_sats = Satellite.objects.filter(
+        associated_satellite__isnull=True
+    ).order_by('-id')[:5].prefetch_related(prefetch_approved, prefetch_suggested)
     # Calculate latest contributors
     latest_data_satellites = []
     found = False
@@ -55,9 +56,8 @@ def home(request):
     page = paginator.page(1)
     while not found:
         for data in page.object_list:
-            if data.satellite.id in latest_data_satellites:
-                continue
-            latest_data_satellites.append(data.satellite.id)
+            if data.satellite.id not in latest_data_satellites:
+                latest_data_satellites.append(data.satellite.id)
             if len(latest_data_satellites) > 5:
                 found = True
                 break
@@ -66,9 +66,11 @@ def home(request):
         else:
             break
 
+    # Check if satellite is merged and if it is then show its associated entry.
     latest_data = Satellite.objects.filter(
-        pk__in=latest_data_satellites
-    ).prefetch_related(prefetch_approved, prefetch_suggested)
+        associated_satellite__isnull=True
+    ).filter(Q(pk__in=latest_data_satellites) | Q(associated_with__pk__in=latest_data_satellites)
+             ).prefetch_related(prefetch_approved, prefetch_suggested)
 
     # Calculate latest contributors
     date_from = now() - timedelta(days=1)
@@ -90,7 +92,9 @@ def transmitters_list(request):
 
     :returns: base/transmitters.html
     """
-    transmitters = Transmitter.objects.prefetch_related('satellite', 'downlink_mode')
+    transmitters = Transmitter.objects.filter(
+        satellite__associated_satellite__isnull=True, satellite__satellite_entry__approved=True
+    ).select_related('satellite', 'satellite__satellite_entry', 'satellite__satellite_identifier')
 
     return render(request, 'base/transmitters.html', {'transmitters': transmitters})
 
@@ -110,23 +114,35 @@ def satellites(request):
 
     :returns: base/satellites.html
     """
-    satellite_objects = Satellite.objects.prefetch_related(
-        'operator',
+    satellite_objects = Satellite.objects.filter(
+        associated_satellite__isnull=True, satellite_entry__approved=True
+    ).select_related('satellite_entry__operator').prefetch_related(
         Prefetch(
             'transmitter_entries',
             queryset=Transmitter.objects.all(),
             to_attr='approved_transmitters'
         )
+    ).annotate(
+        satellite_suggestions_count=Count(
+            'satellite_identifier__satellite_entries',
+            filter=Q(satellite_identifier__satellite_entries__reviewed__isnull=True)
+        )
     )
     return render(request, 'base/satellites.html', {'satellites': satellite_objects})
 
 
-def satellite(request, norad):
+def satellite(request, norad=None, sat_id=None):
     """View to render satellite page.
 
     :returns: base/satellite.html
     """
-    satellite_obj = get_object_or_404(Satellite, norad_cat_id=norad)
+    if norad:
+        satellite_obj = get_object_or_404(Satellite, satellite_entry__norad_cat_id=norad)
+    else:
+        satellite_obj = get_object_or_404(Satellite, satellite_identifier__sat_id=sat_id)
+
+    if satellite_obj.associated_satellite:
+        satellite_obj = satellite_obj.associated_satellite
 
     latest_tle = None
     latest_tle_set = None
@@ -147,17 +163,23 @@ def satellite(request, norad):
         except Transmitter.DoesNotExist:
             suggestion.transmitter = None
 
+    satellite_suggestions = SatelliteSuggestion.objects.filter(
+        satellite_identifier=satellite_obj.satellite_identifier
+    )
+
     try:
-        # pull the last 5 observers and their submission timestamps for this satellite
-        recent_observers = DemodData.objects.filter(satellite=satellite_obj) \
-            .values('observer').annotate(latest_payload=Max('timestamp')) \
-            .order_by('-latest_payload')[:5]
+        # pull the last 5 observers and their submission timestamps for this
+        # satellite and for the satellite that are associated with it
+        recent_observers = DemodData.objects.filter(
+            Q(satellite=satellite_obj) | Q(satellite__associated_satellite=satellite_obj)
+        ).values('observer').annotate(latest_payload=Max('timestamp')
+                                      ).order_by('-latest_payload')[:5]
     except (ObjectDoesNotExist, IndexError):
         recent_observers = ''
 
     # decide whether a map (and map link) will be visible or not (ie: re-entered)
     showmap = False
-    if satellite_obj.status not in ['re-entered', 'future'] and latest_tle:
+    if satellite_obj.satellite_entry.status not in ['re-entered', 'future'] and latest_tle:
         showmap = True
 
     return render(
@@ -165,6 +187,7 @@ def satellite(request, norad):
             'satellite': satellite_obj,
             'latest_tle': latest_tle,
             'transmitter_suggestions': transmitter_suggestions,
+            'satellite_suggestions': satellite_suggestions,
             'mapbox_token': settings.MAPBOX_TOKEN,
             'recent_observers': recent_observers,
             'badge_telemetry_count': millify(satellite_obj.telemetry_data_count),
@@ -174,7 +197,7 @@ def satellite(request, norad):
 
 
 @login_required
-def request_export(request, norad, period=None):
+def request_export(request, sat_pk, period=None):
     """View to request frames export download.
 
     This triggers a request to collect and zip up the requested data for
@@ -182,13 +205,50 @@ def request_export(request, norad, period=None):
     completed.
     :returns: the originating satellite page
     """
-    get_object_or_404(Satellite, norad_cat_id=norad)
-    export_frames.delay(norad, request.user.id, period)
+    satellite_obj = get_object_or_404(Satellite, id=sat_pk)
+    if satellite_obj.associated_satellite:
+        satellite_obj = satellite_obj.associated_satellite
+
+    export_frames.delay(satellite_obj.satellite_identifier.sat_id, request.user.id, period)
     messages.success(
         request, ('Your download request was received. '
                   'You will get an email when it\'s ready')
     )
-    return redirect(reverse('satellite', kwargs={'norad': norad}))
+    return redirect(
+        reverse('satellite', kwargs={'sat_id': satellite_obj.satellite_identifier.sat_id})
+    )
+
+
+@login_required
+@require_POST
+def satellite_suggestion_handler(request):
+    """Returns the Satellite page after approving or rejecting a suggestion if
+    user has approve permission.
+
+    :returns: Satellite page
+    """
+    satellite_entry = get_object_or_404(SatelliteSuggestion, pk=request.POST['pk'])
+    satellite_obj = get_object_or_404(
+        Satellite, satellite_identifier=satellite_entry.satellite_identifier
+    )
+    if request.user.has_perm('base.approve_satellitesuggestion'):
+        if 'approve' in request.POST:
+            satellite_entry.approved = True
+            messages.success(request, ('Satellite approved.'))
+        elif 'reject' in request.POST:
+            satellite_entry.approved = False
+            messages.success(request, ('Satellite rejected.'))
+        satellite_entry.reviewed = now()
+        satellite_entry.reviewer = request.user
+        satellite_entry.save(update_fields=['approved', 'reviewed', 'reviewer'])
+
+        if satellite_entry.approved:
+            satellite_obj.satellite_entry = satellite_entry
+            satellite_obj.save(update_fields=['satellite_entry'])
+    redirect_page = redirect(
+        reverse('satellite', kwargs={'sat_id': satellite_obj.satellite_identifier.sat_id})
+    )
+    return redirect_page
 
 
 @login_required
@@ -200,7 +260,7 @@ def transmitter_suggestion_handler(request):
     :returns: Satellite page
     """
     transmitter = get_object_or_404(TransmitterSuggestion, pk=request.POST['pk'])
-    if request.user.has_perm('base.approve'):
+    if request.user.has_perm('base.approve_transmittersuggestion'):
         if 'approve' in request.POST:
             transmitter.approved = True
             messages.success(request, ('Transmitter approved.'))
@@ -209,11 +269,10 @@ def transmitter_suggestion_handler(request):
             messages.success(request, ('Transmitter rejected.'))
         transmitter.reviewed = now()
         transmitter.reviewer = request.user
-
-        transmitter.save()
+        transmitter.save(update_fields=['approved', 'reviewed', 'reviewer'])
 
     redirect_page = redirect(
-        reverse('satellite', kwargs={'norad': transmitter.satellite.norad_cat_id})
+        reverse('satellite', kwargs={'sat_id': transmitter.satellite.satellite_identifier.sat_id})
     )
     return redirect_page
 
@@ -246,9 +305,14 @@ def search(request):
 
     if query_string:
         results = Satellite.objects.filter(
-            Q(name__icontains=query_string) | Q(names__icontains=query_string)
-            | Q(norad_cat_id__icontains=query_string)  # noqa: W503 google W503 it is evil
-        ).order_by('name').prefetch_related(
+            associated_satellite__isnull=True, satellite_entry__approved=True
+        ).filter(
+            Q(satellite_entry__name__icontains=query_string)
+            | Q(satellite_entry__names__icontains=query_string)
+            | Q(satellite_entry__norad_cat_id__icontains=query_string)
+            | Q(satellite_identifier__sat_id__icontains=query_string)
+            | Q(associated_with__satellite_identifier__sat_id__icontains=query_string)
+        ).order_by('satellite_entry__name').prefetch_related(
             Prefetch(
                 'transmitter_entries',
                 queryset=Transmitter.objects.all(),
@@ -257,7 +321,9 @@ def search(request):
         )
 
     if results.count() == 1:
-        return redirect(reverse('satellite', kwargs={'norad': results[0].norad_cat_id}))
+        return redirect(
+            reverse('satellite', kwargs={'sat_id': results[0].satellite_identifier.sat_id})
+        )
 
     return render(request, 'base/search.html', {'results': results, 'q': query_string})
 
@@ -278,7 +344,7 @@ def stats(request):
         except OperationalError:
             pass
     for sid in ids:
-        stat = cache.get(sid['id'])
+        stat = cache.get(sid)
         cached_satellites.append(stat)
 
     return render(
@@ -328,7 +394,7 @@ class TransmitterCreateView(LoginRequiredMixin, BSModalCreateView):
     """A django-bootstrap-modal-forms view for creating transmitter suggestions"""
     template_name = 'base/modals/transmitter_create.html'
     model = TransmitterEntry
-    form_class = TransmitterModelForm
+    form_class = TransmitterCreateForm
     success_message = 'Your transmitter suggestion was stored successfully and will be \
                        reviewed by a moderator. Thanks for contibuting!'
 
@@ -353,7 +419,9 @@ class TransmitterCreateView(LoginRequiredMixin, BSModalCreateView):
         transmitter.created_by = self.user
         # Prevents sending notification twice as form_valid is triggered for validation and saving
         if not self.request.is_ajax():
-            notify_transmitter_suggestion.delay(transmitter.satellite.id, self.user.id)
+            notify_suggestion.delay(
+                transmitter.satellite.satellite_entry.id, self.user.id, 'transmitter'
+            )
         return super().form_valid(form)
 
     def get_success_url(self):
@@ -366,7 +434,7 @@ class TransmitterUpdateView(LoginRequiredMixin, BSModalUpdateView):
     model = TransmitterEntry
     form_class = TransmitterUpdateForm
     success_message = 'Your transmitter suggestion was stored successfully and will be \
-                       reviewed by a moderator. Thanks for contibuting!'
+                       reviewed by a moderator. Thanks for contributing!'
 
     user = get_user_model()
 
@@ -385,20 +453,115 @@ class TransmitterUpdateView(LoginRequiredMixin, BSModalUpdateView):
         transmitter.created_by = self.user
         # Prevents sending notification twice as form_valid is triggered for validation and saving
         if not self.request.is_ajax():
-            notify_transmitter_suggestion.delay(transmitter.satellite.id, self.user.id)
+            notify_suggestion.delay(
+                transmitter.satellite.satellite_entry.id, self.user.id, 'transmitter'
+            )
         return super().form_valid(form)
 
     def get_success_url(self):
         return self.request.META.get('HTTP_REFERER')
 
 
-class SatelliteUpdateView(PermissionRequiredMixin, BSModalUpdateView):
-    """A django-bootstrap-modal-forms view for updating satellite fields"""
-    permission_required = 'base.change_satellite'
-    model = Satellite
+@login_required
+@require_POST
+def merge_satellites(request):
+    """Merges satellites if user has merge permission.
+    """
+    primary_satellite = get_object_or_404(Satellite, pk=request.POST['primary-satellite'])
+    associated_satellite = get_object_or_404(Satellite, pk=request.POST['associated-satellite'])
+
+    if request.user.has_perm('base.merge_satellites'):
+        if primary_satellite.pk == associated_satellite.pk:
+            messages.error(request, ('Primary and Associated satellites are the same!'))
+        else:
+            associated_satellite.associated_satellite = primary_satellite
+            associated_satellite.save(update_fields=['associated_satellite'])
+            messages.success(request, ('Merge is complete!'))
+    else:
+        messages.error(request, ('No permission to merge satellites!'))
+    return redirect(reverse('satellites'))
+
+
+class SatelliteCreateView(LoginRequiredMixin, BSModalCreateView):
+    """A django-bootstrap-modal-forms view for creating satellite suggestions"""
+    template_name = 'base/modals/satellite_create.html'
+    model = SatelliteEntry
+    form_class = SatelliteCreateForm
+    success_message = 'Your satellite suggestion was stored successfully and will be \
+                       reviewed by a moderator. Thanks for contributing!'
+
+    user = get_user_model()
+
+    def dispatch(self, request, *args, **kwargs):
+        self.user = request.user
+        return super().dispatch(request, *args, **kwargs)
+
+    def form_valid(self, form):
+        satellite_entry = form.instance
+        satellite_obj = None
+        # Create Satellite Identifier only when POST request is for saving and
+        # NORAD ID is not used by other Satellite.
+        if not self.request.is_ajax():
+            try:
+                satellite_obj = Satellite.objects.get(
+                    satellite_entry__norad_cat_id=satellite_entry.norad_cat_id
+                )
+                satellite_entry.satellite_identifier = satellite_obj.satellite_identifier
+            except Satellite.DoesNotExist:
+                satellite_entry.satellite_identifier = SatelliteIdentifier.objects.create()
+
+        satellite_entry.created = now()
+        satellite_entry.created_by = self.user
+
+        # form_valid triggers also save() allowing us to use satellite_entry
+        # for creating Satellite object, see comment bellow.
+        response = super().form_valid(form)
+
+        # Prevents sending notification twice as form_valid is triggered for
+        # validation and saving. Also create and Satellite object only when POST
+        # request is for saving and NORAD ID is not used by other Satellite.
+        if not self.request.is_ajax():
+            if not satellite_obj:
+                satellite_obj = Satellite.objects.create(
+                    satellite_identifier=satellite_entry.satellite_identifier,
+                    satellite_entry=satellite_entry
+                )
+            notify_suggestion.delay(satellite_obj.satellite_entry.pk, self.user.id, 'satellite')
+
+        return response
+
+    def get_success_url(self):
+        return self.request.META.get('HTTP_REFERER')
+
+
+class SatelliteUpdateView(LoginRequiredMixin, BSModalUpdateView):
+    """A django-bootstrap-modal-forms view for updating satellite entries"""
     template_name = 'base/modals/satellite_update.html'
-    form_class = SatelliteModelForm
-    success_message = 'Satellite was updated.'
+    model = SatelliteEntry
+    form_class = SatelliteUpdateForm
+    success_message = 'Your satellite suggestion was stored successfully and will be \
+                       reviewed by a moderator. Thanks for contributing!'
+
+    user = get_user_model()
+
+    def dispatch(self, request, *args, **kwargs):
+        self.user = request.user
+        return super().dispatch(request, *args, **kwargs)
+
+    def form_valid(self, form):
+        satellite_entry = form.instance
+        initial_satellite_entry_pk = satellite_entry.pk
+        # Add update as a new SatelliteEntry object and change fields in order to be a suggestion
+        satellite_entry.pk = None
+        satellite_entry.reviewed = None
+        satellite_entry.reviewer = None
+        satellite_entry.approved = False
+        satellite_entry.created = now()
+        satellite_entry.created_by = self.user
+        # Prevents sending notification twice as form_valid is triggered for validation and saving
+        if not self.request.is_ajax():
+            notify_suggestion.delay(initial_satellite_entry_pk, self.user.id, 'satellite')
+        return super().form_valid(form)
 
     def get_success_url(self):
         return self.request.META.get('HTTP_REFERER')
