@@ -55,13 +55,12 @@ LOCAL_APPS = (
 if DEBUG:
     DJANGO_APPS += ('debug_toolbar', )
     DEBUG_TOOLBAR_CONFIG = {
-        'SHOW_TOOLBAR_CALLBACK': lambda request: request.environ.get('SERVER_NAME', None) !=
+        'SHOW_TOOLBAR_CALLBACK': lambda request: request.environ.get('SERVER_NAME', None) !=  # noqa W504 pylint: disable=C0301
         'testserver',
     }
 
 if AUTH0:
     THIRD_PARTY_APPS += ('social_django', )
-    # LOCAL_APPS += ('auth0login', )
 
 INSTALLED_APPS = DJANGO_APPS + THIRD_PARTY_APPS + LOCAL_APPS
 
@@ -187,7 +186,6 @@ WSGI_APPLICATION = 'db.wsgi.application'
 # Auth
 AUTHENTICATION_BACKENDS = ('django.contrib.auth.backends.ModelBackend', )
 if AUTH0:
-    # AUTHENTICATION_BACKENDS += ('auth0login.auth0backend.Auth0', )
     AUTHENTICATION_BACKENDS += ('social_core.backends.auth0.Auth0OAuth2', )
 
 ACCOUNT_AUTHENTICATION_METHOD = 'username'
@@ -245,7 +243,7 @@ LOGGING = {
 # Sentry
 SENTRY_ENABLED = config('SENTRY_ENABLED', default=False, cast=bool)
 if SENTRY_ENABLED:
-    sentry_sdk.init(
+    sentry_sdk.init(  # pylint: disable=abstract-class-instantiated
         environment=ENVIRONMENT,
         dsn=config('SENTRY_DSN', default=''),
         release='satnogs-db@{}'.format(__version__),
@@ -328,7 +326,10 @@ SPECTACULAR_SETTINGS = {
             'description': 'Production server'
         }
     ],
-    'PREPROCESSING_HOOKS': ['drf_spectacular.hooks.preprocess_exclude_path_format'],
+    'PREPROCESSING_HOOKS': [
+        'drf_spectacular.hooks.preprocess_exclude_path_format',
+        'db.api.drf_spectacular_hooks.exclude_paths_hook'
+    ],
     'POSTPROCESSING_HOOKS': ['drf_spectacular.hooks.postprocess_schema_enums'],
     'GET_MOCK_REQUEST': 'drf_spectacular.plumbing.build_mock_request',
 
@@ -506,3 +507,6 @@ if ENVIRONMENT == 'dev':
 
 # for h5 artifact uploads
 DATA_UPLOAD_MAX_MEMORY_SIZE = 5242880
+
+# Django 3.2 allows for configurable db keys, we just need auto
+DEFAULT_AUTO_FIELD = 'django.db.models.AutoField'
