@@ -28,6 +28,10 @@ $(document).ready(function() {
             alerticon = 'far fa-thumbs-up';
             alerttitle = 'Success';
         }
+        if ($(this).data('alertclass') == 'alert-error') {
+            alerticon = 'fas fa-ban';
+            alerttitle = 'Error';
+        }
         if ($(this).data('alertclass') == 'alert-warning') {
             alerticon = 'fas fa-exclamation';
             alerttitle = 'Alert';
